@@ -1,30 +1,56 @@
+#ifdef _OPENMP
+  #include <omp.h>
+  #define OMP_NUM_PROCS omp_get_num_procs()
+  #define OMP_THREAD_LIMIT omp_get_thread_limit()
+  #define OMP_MAX_THREADS omp_get_max_threads()
+#else
+  #define OMP_NUM_PROCS 1
+  #define OMP_THREAD_LIMIT 1
+  #define OMP_MAX_THREADS 1
+#endif
+
 #include <R.h>
 #include <Rinternals.h>
 
 #define SEXPPTR(x) ((SEXP *)DATAPTR(x))  // to avoid overhead of looped VECTOR_ELT
+#define SEXPPTR_RO(x) ((const SEXP *)DATAPTR_RO(x))  // to avoid overhead of looped VECTOR_ELT
+
 #define NISNAN(x) ((x) == (x))  // opposite of ISNAN for doubles
 // Faster than Rinternals version (which uses math library version)
 #undef ISNAN
 #define ISNAN(x) ((x) != (x))
 
-void matCopyAttr(SEXP out, SEXP x, SEXP Rdrop, int ng);
-void DFcopyAttr(SEXP out, SEXP x, int ng);
+extern int max_threads;
 
-void multi_yw(void *, void *, void *, void *, void *, void *, void *, void *, void *, void *);
-SEXP collapse_init(SEXP);
-SEXP dt_na(SEXP, SEXP);
-SEXP allNAv(SEXP, SEXP);
+// from base_radixsort.h (with significant modifications)
 SEXP Cradixsort(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
-SEXP frankds(SEXP, SEXP, SEXP, SEXP);
+void num1radixsort(int *, Rboolean, Rboolean, SEXP);
+void iradixsort(int *, Rboolean, Rboolean, int, int *);
+void dradixsort(int *, Rboolean, Rboolean, int, double *);
+
+// from stats_mAR.c
+void multi_yw(void *, void *, void *, void *, void *, void *, void *, void *, void *, void *);
 SEXP pacf1(SEXP, SEXP);
+
+// from data.table.h (with major modifications)
+SEXP collapse_init(SEXP);
+SEXP dt_na(SEXP, SEXP, SEXP, SEXP);
+SEXP allNAv(SEXP, SEXP);
+SEXP frankds(SEXP, SEXP, SEXP, SEXP);
 SEXP rbindlist(SEXP, SEXP, SEXP, SEXP);
 SEXP setcolorder(SEXP, SEXP);
 SEXP subsetDT(SEXP, SEXP, SEXP, SEXP);
 SEXP subsetCols(SEXP, SEXP, SEXP);
 SEXP subsetVector(SEXP, SEXP, SEXP);
-SEXP Calloccol(SEXP dt); // , SEXP Rn
+SEXP Calloccol(SEXP);
+
+// Native collapse functions
+void matCopyAttr(SEXP out, SEXP x, SEXP Rdrop, int ng);
+void DFcopyAttr(SEXP out, SEXP x, int ng);
 SEXP falloc(SEXP, SEXP);
 SEXP frange(SEXP x, SEXP Rnarm);
+SEXP fdist(SEXP x, SEXP vec, SEXP Rret, SEXP Rnthreads);
+SEXP fnrowC(SEXP x);
 // SEXP CasChar(SEXP x);
 SEXP setAttributes(SEXP x, SEXP a);
 SEXP setattributes(SEXP x, SEXP a);
@@ -74,17 +100,17 @@ SEXP flastC(SEXP x, SEXP Rng, SEXP g, SEXP Rnarm);
 SEXP flastmC(SEXP x, SEXP Rng, SEXP g, SEXP Rnarm, SEXP Rdrop);
 SEXP flastlC(SEXP x, SEXP Rng, SEXP g, SEXP Rnarm);
 // fsum rewritten in C:
-SEXP fsumC(SEXP x, SEXP Rng, SEXP g, SEXP w, SEXP Rnarm, SEXP fill, SEXP Rnth);
-SEXP fsummC(SEXP x, SEXP Rng, SEXP g, SEXP w, SEXP Rnarm, SEXP fill, SEXP Rdrop, SEXP Rnth);
-SEXP fsumlC(SEXP x, SEXP Rng, SEXP g, SEXP w, SEXP Rnarm, SEXP fill, SEXP Rdrop, SEXP Rnth);
+SEXP fsumC(SEXP x, SEXP Rng, SEXP g, SEXP w, SEXP Rnarm, SEXP fill, SEXP Rnthreads);
+SEXP fsummC(SEXP x, SEXP Rng, SEXP g, SEXP w, SEXP Rnarm, SEXP fill, SEXP Rdrop, SEXP Rnthreads);
+SEXP fsumlC(SEXP x, SEXP Rng, SEXP g, SEXP w, SEXP Rnarm, SEXP fill, SEXP Rdrop, SEXP Rnthreads);
 // fprod rewritten in C:
 SEXP fprodC(SEXP x, SEXP Rng, SEXP g, SEXP w, SEXP Rnarm);
 SEXP fprodmC(SEXP x, SEXP Rng, SEXP g, SEXP w, SEXP Rnarm, SEXP Rdrop);
 SEXP fprodlC(SEXP x, SEXP Rng, SEXP g, SEXP w, SEXP Rnarm, SEXP Rdrop);
 // fmean rewritten in C:
-SEXP fmeanC(SEXP x, SEXP Rng, SEXP g, SEXP gs, SEXP w, SEXP Rnarm, SEXP Rnth);
-SEXP fmeanmC(SEXP x, SEXP Rng, SEXP g, SEXP gs, SEXP w, SEXP Rnarm, SEXP Rdrop, SEXP Rnth);
-SEXP fmeanlC(SEXP x, SEXP Rng, SEXP g, SEXP gs, SEXP w, SEXP Rnarm, SEXP Rdrop, SEXP Rnth);
+SEXP fmeanC(SEXP x, SEXP Rng, SEXP g, SEXP gs, SEXP w, SEXP Rnarm, SEXP Rnthreads);
+SEXP fmeanmC(SEXP x, SEXP Rng, SEXP g, SEXP gs, SEXP w, SEXP Rnarm, SEXP Rdrop, SEXP Rnthreads);
+SEXP fmeanlC(SEXP x, SEXP Rng, SEXP g, SEXP gs, SEXP w, SEXP Rnarm, SEXP Rdrop, SEXP Rnthreads);
 // fmin and fmax rewritten in C:
 SEXP fminC(SEXP x, SEXP Rng, SEXP g, SEXP Rnarm);
 SEXP fminmC(SEXP x, SEXP Rng, SEXP g, SEXP Rnarm, SEXP Rdrop);
@@ -108,4 +134,28 @@ SEXP fndistinctmC(SEXP x, SEXP g, SEXP Rnarm, SEXP Rdrop, SEXP Rnthreads);
 SEXP fmodeC(SEXP x, SEXP g, SEXP w, SEXP Rnarm, SEXP Rret, SEXP Rnthreads);
 SEXP fmodelC(SEXP x, SEXP g, SEXP w, SEXP Rnarm, SEXP Rret, SEXP Rnthreads);
 SEXP fmodemC(SEXP x, SEXP g, SEXP w, SEXP Rnarm, SEXP Rdrop, SEXP Rret, SEXP Rnthreads);
+// fnth, rewritten in C:
+SEXP fnthC(SEXP x, SEXP p, SEXP g, SEXP w, SEXP Rnarm, SEXP Rret, SEXP Rnthreads, SEXP o, SEXP checko);
+SEXP fnthlC(SEXP x, SEXP p, SEXP g, SEXP w, SEXP Rnarm, SEXP Rdrop, SEXP Rret, SEXP Rnthreads);
+SEXP fnthmC(SEXP x, SEXP p, SEXP g, SEXP w, SEXP Rnarm, SEXP Rdrop, SEXP Rret, SEXP Rnthreads);
+// New: fquantile:
+SEXP fquantileC(SEXP x, SEXP Rprobs, SEXP w, SEXP o, SEXP Rnarm, SEXP Rtype, SEXP Rnames, SEXP checko);
+// Helper functions for C API
+double dquickselect_elem(double *x, const int n, const unsigned int elem, double h);
+double iquickselect_elem(int *x, const int n, const unsigned int elem, double h);
+double dquickselect(double *x, const int n, const int ret, const double Q);
+double iquickselect(int *x, const int n, const int ret, const double Q);
+double nth_int(const int *restrict px, const int *restrict po, const int l, const int sorted, const int narm, const int ret, const double Q);
+double nth_double(const double *restrict px, const int *restrict po, const int l, const int sorted, const int narm, const int ret, const double Q);
+double nth_int_ord(const int *restrict px, const int *restrict po, int l, const int narm, const int ret, const double Q);
+double nth_double_ord(const double *restrict px, const int *restrict po, int l, const int narm, const int ret, const double Q);
+double w_nth_int_ord(const int *restrict px, const double *restrict pw, const int *restrict po, double h, int l, const int narm, const int ret, const double Q);
+double w_nth_double_ord(const double *restrict px, const double *restrict pw, const int *restrict po, double h, int l, const int narm, const int ret, const double Q);
+double w_nth_int_qsort(const int *restrict px, const double *restrict pw, const int *restrict po, double h,
+                       const int l, const int sorted, const int narm, const int ret, const double Q);
+double w_nth_double_qsort(const double *restrict px, const double *restrict pw, const int *restrict po, double h,
+                          const int l, const int sorted, const int narm, const int ret, const double Q);
+SEXP nth_impl(SEXP x, int narm, int ret, double Q);
+SEXP nth_ord_impl(SEXP x, int *pxo, int narm, int ret, double Q);
+SEXP w_nth_ord_impl(SEXP x, int *pxo, double *pw, int narm, int ret, double Q, double h);
 
