@@ -1,6 +1,3 @@
-#ifdef _OPENMP
-#include <omp.h>
-#endif
 #include "collapse_c.h"
 
 /*
@@ -178,7 +175,7 @@ if(probs[0] == 0.0 || probs[np-1] == 1.0) {                                   \
 */
 #define FQUANTILE_CORE(QFUN)                                                  \
   double h, Q;                                                                \
-  int ih;                                                                     \
+  int ih = 0; /* To avoid -Wmaybe-uninitialized */                            \
   for(int i = 0, offset = 0; i < np; ++i) {                                   \
     Q = probs[i];                                                             \
     if(Q > 0.0 && Q < 1.0) {                                                  \
@@ -460,7 +457,7 @@ SEXP fquantileC(SEXP x, SEXP Rprobs, SEXP w, SEXP o, SEXP Rnarm, SEXP Rtype, SEX
 double dquickselect(double *x, const int n, const int ret, const double Q) {
   if(n == 0) return NA_REAL;
   unsigned int elem;
-  double a, b, h;
+  double a, b, h = 0.0; /* To avoid -Wmaybe-uninitialized */
   RETQSWITCH(n);
   elem = h; h -= elem; // need to subtract elem
   QUICKSELECT(dswap);
@@ -475,7 +472,7 @@ double iquickselect(int *x, const int n, const int ret, const double Q) {
   if(n == 0) return NA_REAL;
   unsigned int elem;
   int a, b;
-  double h;
+  double h = 0.0; /* To avoid -Wmaybe-uninitialized */
   RETQSWITCH(n);
   elem = h; h -= elem; // need to subtract elem
   QUICKSELECT(iswap);
@@ -491,7 +488,7 @@ double iquickselect(int *x, const int n, const int ret, const double Q) {
 
 // Expects pw and po to be consistent
 double w_compute_h(const double *pw, const int *po, const int l, const int sorted, const int ret, const double Q) {
-  double sumw = 0.0, mu, h;
+  double sumw = 0.0, mu, h = 0.0; /* To avoid -Wmaybe-uninitialized */
   int nw0 = 0;
   if(sorted) {
     for(int i = 0; i != l; ++i) {
@@ -583,7 +580,7 @@ if(ret < 3) { /* lower (2), or average (1) element*/                         \
 // Expects pointer px to be decremented by 1
 #undef NTH_ORDVEC
 #define NTH_ORDVEC                                                         \
-double a, b, h;                                                            \
+double a, b, h = 0.0; /* To avoid -Wmaybe-uninitialized */                 \
 RETQSWITCH(l);                                                             \
 int ih = h; a = px[po[ih]]; h -= ih;                                       \
 if((ret < 4 && (ret != 1 || l%2 == 1)) || ih == l-1 || h <= 0.0) return a; \
@@ -1251,7 +1248,7 @@ int Rties2int(SEXP x) {
 #undef CHECK_GROUPS
 #define CHECK_GROUPS(nrx, cond)                                                                                \
 if(TYPEOF(g) != VECSXP || !inherits(g, "GRP")) error("g needs to be an object of class 'GRP', see ?GRP");      \
-const SEXP *restrict pg = SEXPPTR(g), ord = pg[6];                                                             \
+const SEXP *restrict pg = SEXPPTR_RO(g), ord = pg[6];                                                             \
 ng = INTEGER(pg[0])[0];                                                                                        \
 int sorted = LOGICAL(pg[5])[1] == 1, *restrict pgs = INTEGER(pg[2]), *restrict po, *restrict pst, maxgrpn = 0; \
 if(nrx != length(pg[1])) error("length(g) must match nrow(x)");                                                \
@@ -1429,7 +1426,8 @@ SEXP fnthlC(SEXP x, SEXP p, SEXP g, SEXP w, SEXP Rnarm, SEXP Rdrop, SEXP Rret, S
   if(l < 1) return x;
   if(nthreads > max_threads) nthreads = max_threads;
 
-  SEXP out = PROTECT(allocVector(nullg && drop ? REALSXP : VECSXP, l)), *restrict px = SEXPPTR(x);
+  SEXP out = PROTECT(allocVector(nullg && drop ? REALSXP : VECSXP, l));
+  const SEXP *restrict px = SEXPPTR_RO(x);
   int nrx = length(px[0]);
 
   CHECK_PROB(nrx);
