@@ -47,12 +47,6 @@ wsum <- function(x, w, na.rm = FALSE) {
   bsum(x*w)
 }
 
-wBY <- function(x, f, FUN, w, ...) {
-  if(is.atomic(x) && !is.array(x)) return(mapply(FUN, split(x, f), split(w, f), ...))
-  wspl <- split(w, f)
-  if(is.atomic(x)) return(dapply(x, function(xi) mapply(FUN, split(xi, f), wspl, ...)))
-  qDF(dapply(x, function(xi) mapply(FUN, split(xi, f), wspl, ...), return = "matrix"))
-}
 
 for (nth in 1:2) {
 
@@ -125,18 +119,18 @@ test_that("fsum with weights performs like wsum (defined above)", {
   expect_equal(fsum(mtcars, w = wdat, na.rm = FALSE), dapply(mtcars, wsum, wdat))
   expect_equal(fsum(mtcNA, w = wdat, na.rm = FALSE), dapply(mtcNA, wsum, wdat))
   expect_equal(fsum(mtcNA, w = wdat), dapply(mtcNA, wsum, wdat, na.rm = TRUE))
-  expect_equal(fsum(x, f, w), wBY(x, f, wsum, w))
-  expect_equal(fsum(x, f, w, na.rm = FALSE), wBY(x, f, wsum, w))
-  expect_equal(fsum(xNA, f, w, na.rm = FALSE), wBY(xNA, f, wsum, w))
-  expect_equal(fsum(xNA, f, w), wBY(xNA, f, wsum, w, na.rm = TRUE))
-  expect_equal(fsum(m, g, wdat), wBY(m, gf, wsum, wdat))
-  expect_equal(fsum(m, g, wdat, na.rm = FALSE), wBY(m, gf, wsum, wdat))
-  expect_equal(fsum(mNA, g, wdat, na.rm = FALSE),  wBY(mNA, gf, wsum, wdat))
-  expect_equal(fsum(mNA, g, wdat), condan20(wBY(mNA, gf, wsum, wdat, na.rm = TRUE), fill_arg == 2L))
-  expect_equal(fsum(mtcars, g, wdat), wBY(mtcars, gf, wsum, wdat))
-  expect_equal(fsum(mtcars, g, wdat, na.rm = FALSE), wBY(mtcars, gf, wsum, wdat))
-  expect_equal(fsum(mtcNA, g, wdat, na.rm = FALSE), wBY(mtcNA, gf, wsum, wdat))
-  expect_equal(fsum(mtcNA, g, wdat), condan20(wBY(mtcNA, gf, wsum, wdat, na.rm = TRUE), fill_arg == 2L))
+  expect_equal(fsum(x, f, w), BY(x, f, wsum, w))
+  expect_equal(fsum(x, f, w, na.rm = FALSE), BY(x, f, wsum, w))
+  expect_equal(fsum(xNA, f, w, na.rm = FALSE), BY(xNA, f, wsum, w))
+  expect_equal(fsum(xNA, f, w), BY(xNA, f, wsum, w, na.rm = TRUE))
+  expect_equal(fsum(m, g, wdat), BY(m, gf, wsum, wdat))
+  expect_equal(fsum(m, g, wdat, na.rm = FALSE), BY(m, gf, wsum, wdat))
+  expect_equal(fsum(mNA, g, wdat, na.rm = FALSE),  BY(mNA, gf, wsum, wdat))
+  expect_equal(fsum(mNA, g, wdat), condan20(BY(mNA, gf, wsum, wdat, na.rm = TRUE), fill_arg == 2L))
+  expect_equal(fsum(mtcars, g, wdat), BY(mtcars, gf, wsum, wdat))
+  expect_equal(fsum(mtcars, g, wdat, na.rm = FALSE), BY(mtcars, gf, wsum, wdat))
+  expect_equal(fsum(mtcNA, g, wdat, na.rm = FALSE), BY(mtcNA, gf, wsum, wdat))
+  expect_equal(fsum(mtcNA, g, wdat), condan20(BY(mtcNA, gf, wsum, wdat, na.rm = TRUE), fill_arg == 2L))
   # missing weights
   expect_equal(fsum(NA, w = NA), if(fill_arg == 1L) NA_real_ else 0)
   expect_equal(fsum(NA, w = NA, na.rm = FALSE), wsum(NA, NA))
@@ -159,18 +153,18 @@ test_that("fsum with weights performs like wsum (defined above)", {
   expect_equal(fsum(mtcars, w = wdatNA, na.rm = FALSE), dapply(mtcars, wsum, wdatNA))
   expect_equal(fsum(mtcNA, w = wdatNA, na.rm = FALSE), dapply(mtcNA, wsum, wdatNA))
   expect_equal(fsum(mtcNA, w = wdatNA), dapply(mtcNA, wsum, wdatNA, na.rm = TRUE))
-  expect_equal(fsum(x, f, wNA), wBY(x, f, wsum, wNA, na.rm = TRUE))
-  expect_equal(fsum(x, f, wNA, na.rm = FALSE), wBY(x, f, wsum, wNA))
-  expect_equal(fsum(xNA, f, wNA, na.rm = FALSE), wBY(xNA, f, wsum, wNA))
-  expect_equal(fsum(xNA, f, wNA), wBY(xNA, f, wsum, wNA, na.rm = TRUE))
-  expect_equal(fsum(m, g, wdatNA), wBY(m, gf, wsum, wdatNA, na.rm = TRUE))
-  expect_equal(fsum(m, g, wdatNA, na.rm = FALSE), wBY(m, gf, wsum, wdatNA))
-  expect_equal(fsum(mNA, g, wdatNA, na.rm = FALSE),  wBY(mNA, gf, wsum, wdatNA))
-  expect_equal(fsum(mNA, g, wdatNA), condan20(wBY(mNA, gf, wsum, wdatNA, na.rm = TRUE), fill_arg == 2L))
-  expect_equal(fsum(mtcars, g, wdatNA), wBY(mtcars, gf, wsum, wdatNA, na.rm = TRUE))
-  expect_equal(fsum(mtcars, g, wdatNA, na.rm = FALSE), wBY(mtcars, gf, wsum, wdatNA))
-  expect_equal(fsum(mtcNA, g, wdatNA, na.rm = FALSE), wBY(mtcNA, gf, wsum, wdatNA))
-  expect_equal(fsum(mtcNA, g, wdatNA), condan20(wBY(mtcNA, gf, wsum, wdatNA, na.rm = TRUE), fill_arg == 2L))
+  expect_equal(fsum(x, f, wNA), BY(x, f, wsum, wNA, na.rm = TRUE))
+  expect_equal(fsum(x, f, wNA, na.rm = FALSE), BY(x, f, wsum, wNA))
+  expect_equal(fsum(xNA, f, wNA, na.rm = FALSE), BY(xNA, f, wsum, wNA))
+  expect_equal(fsum(xNA, f, wNA), BY(xNA, f, wsum, wNA, na.rm = TRUE))
+  expect_equal(fsum(m, g, wdatNA), BY(m, gf, wsum, wdatNA, na.rm = TRUE))
+  expect_equal(fsum(m, g, wdatNA, na.rm = FALSE), BY(m, gf, wsum, wdatNA))
+  expect_equal(fsum(mNA, g, wdatNA, na.rm = FALSE),  BY(mNA, gf, wsum, wdatNA))
+  expect_equal(fsum(mNA, g, wdatNA), condan20(BY(mNA, gf, wsum, wdatNA, na.rm = TRUE), fill_arg == 2L))
+  expect_equal(fsum(mtcars, g, wdatNA), BY(mtcars, gf, wsum, wdatNA, na.rm = TRUE))
+  expect_equal(fsum(mtcars, g, wdatNA, na.rm = FALSE), BY(mtcars, gf, wsum, wdatNA))
+  expect_equal(fsum(mtcNA, g, wdatNA, na.rm = FALSE), BY(mtcNA, gf, wsum, wdatNA))
+  expect_equal(fsum(mtcNA, g, wdatNA), condan20(BY(mtcNA, gf, wsum, wdatNA, na.rm = TRUE), fill_arg == 2L))
 })
 
 test_that("fsum performs numerically stable", {
@@ -401,18 +395,18 @@ test_that("fsum with integers and weights performs like wsum (defined above)", {
   expect_equal(fsum(mtcars, w = wdat, na.rm = FALSE), dapply(mtcars, wsum, wdat))
   expect_equal(fsum(mtcNA, w = wdat, na.rm = FALSE), dapply(mtcNA, wsum, wdat))
   expect_equal(fsum(mtcNA, w = wdat), dapply(mtcNA, wsum, wdat, na.rm = TRUE))
-  expect_equal(fsum(x, f, w), wBY(x, f, wsum, w))
-  expect_equal(fsum(x, f, w, na.rm = FALSE), wBY(x, f, wsum, w))
-  expect_equal(fsum(xNA, f, w, na.rm = FALSE), wBY(xNA, f, wsum, w))
-  expect_equal(fsum(xNA, f, w), wBY(xNA, f, wsum, w, na.rm = TRUE))
-  expect_equal(fsum(m, g, wdat), wBY(m, gf, wsum, wdat))
-  expect_equal(fsum(m, g, wdat, na.rm = FALSE), wBY(m, gf, wsum, wdat))
-  expect_equal(fsum(mNA, g, wdat, na.rm = FALSE),  wBY(mNA, gf, wsum, wdat))
-  expect_equal(fsum(mNA, g, wdat), condan20(wBY(mNA, gf, wsum, wdat, na.rm = TRUE), fill_arg == 2L))
-  expect_equal(fsum(mtcars, g, wdat), wBY(mtcars, gf, wsum, wdat))
-  expect_equal(fsum(mtcars, g, wdat, na.rm = FALSE), wBY(mtcars, gf, wsum, wdat))
-  expect_equal(fsum(mtcNA, g, wdat, na.rm = FALSE), wBY(mtcNA, gf, wsum, wdat))
-  expect_equal(fsum(mtcNA, g, wdat), condan20(wBY(mtcNA, gf, wsum, wdat, na.rm = TRUE), fill_arg == 2L))
+  expect_equal(fsum(x, f, w), BY(x, f, wsum, w))
+  expect_equal(fsum(x, f, w, na.rm = FALSE), BY(x, f, wsum, w))
+  expect_equal(fsum(xNA, f, w, na.rm = FALSE), BY(xNA, f, wsum, w))
+  expect_equal(fsum(xNA, f, w), BY(xNA, f, wsum, w, na.rm = TRUE))
+  expect_equal(fsum(m, g, wdat), BY(m, gf, wsum, wdat))
+  expect_equal(fsum(m, g, wdat, na.rm = FALSE), BY(m, gf, wsum, wdat))
+  expect_equal(fsum(mNA, g, wdat, na.rm = FALSE),  BY(mNA, gf, wsum, wdat))
+  expect_equal(fsum(mNA, g, wdat), condan20(BY(mNA, gf, wsum, wdat, na.rm = TRUE), fill_arg == 2L))
+  expect_equal(fsum(mtcars, g, wdat), BY(mtcars, gf, wsum, wdat))
+  expect_equal(fsum(mtcars, g, wdat, na.rm = FALSE), BY(mtcars, gf, wsum, wdat))
+  expect_equal(fsum(mtcNA, g, wdat, na.rm = FALSE), BY(mtcNA, gf, wsum, wdat))
+  expect_equal(fsum(mtcNA, g, wdat), condan20(BY(mtcNA, gf, wsum, wdat, na.rm = TRUE), fill_arg == 2L))
   # missing weights
   expect_equal(fsum(x, w = wNA), wsum(x, wNA, na.rm = TRUE))
   expect_equal(fsum(x, w = wNA, na.rm = FALSE), wsum(x, wNA))
@@ -427,18 +421,18 @@ test_that("fsum with integers and weights performs like wsum (defined above)", {
   expect_equal(fsum(mtcars, w = wdatNA, na.rm = FALSE), dapply(mtcars, wsum, wdatNA))
   expect_equal(fsum(mtcNA, w = wdatNA, na.rm = FALSE), dapply(mtcNA, wsum, wdatNA))
   expect_equal(fsum(mtcNA, w = wdatNA), dapply(mtcNA, wsum, wdatNA, na.rm = TRUE))
-  expect_equal(fsum(x, f, wNA), wBY(x, f, wsum, wNA, na.rm = TRUE))
-  expect_equal(fsum(x, f, wNA, na.rm = FALSE), wBY(x, f, wsum, wNA))
-  expect_equal(fsum(xNA, f, wNA, na.rm = FALSE), wBY(xNA, f, wsum, wNA))
-  expect_equal(fsum(xNA, f, wNA), wBY(xNA, f, wsum, wNA, na.rm = TRUE))
-  expect_equal(fsum(m, g, wdatNA), wBY(m, gf, wsum, wdatNA, na.rm = TRUE))
-  expect_equal(fsum(m, g, wdatNA, na.rm = FALSE), wBY(m, gf, wsum, wdatNA))
-  expect_equal(fsum(mNA, g, wdatNA, na.rm = FALSE),  wBY(mNA, gf, wsum, wdatNA))
-  expect_equal(fsum(mNA, g, wdatNA), condan20(wBY(mNA, gf, wsum, wdatNA, na.rm = TRUE), fill_arg == 2L))
-  expect_equal(fsum(mtcars, g, wdatNA), wBY(mtcars, gf, wsum, wdatNA, na.rm = TRUE))
-  expect_equal(fsum(mtcars, g, wdatNA, na.rm = FALSE), wBY(mtcars, gf, wsum, wdatNA))
-  expect_equal(fsum(mtcNA, g, wdatNA, na.rm = FALSE), wBY(mtcNA, gf, wsum, wdatNA))
-  expect_equal(fsum(mtcNA, g, wdatNA), condan20(wBY(mtcNA, gf, wsum, wdatNA, na.rm = TRUE), fill_arg == 2L))
+  expect_equal(fsum(x, f, wNA), BY(x, f, wsum, wNA, na.rm = TRUE))
+  expect_equal(fsum(x, f, wNA, na.rm = FALSE), BY(x, f, wsum, wNA))
+  expect_equal(fsum(xNA, f, wNA, na.rm = FALSE), BY(xNA, f, wsum, wNA))
+  expect_equal(fsum(xNA, f, wNA), BY(xNA, f, wsum, wNA, na.rm = TRUE))
+  expect_equal(fsum(m, g, wdatNA), BY(m, gf, wsum, wdatNA, na.rm = TRUE))
+  expect_equal(fsum(m, g, wdatNA, na.rm = FALSE), BY(m, gf, wsum, wdatNA))
+  expect_equal(fsum(mNA, g, wdatNA, na.rm = FALSE),  BY(mNA, gf, wsum, wdatNA))
+  expect_equal(fsum(mNA, g, wdatNA), condan20(BY(mNA, gf, wsum, wdatNA, na.rm = TRUE), fill_arg == 2L))
+  expect_equal(fsum(mtcars, g, wdatNA), BY(mtcars, gf, wsum, wdatNA, na.rm = TRUE))
+  expect_equal(fsum(mtcars, g, wdatNA, na.rm = FALSE), BY(mtcars, gf, wsum, wdatNA))
+  expect_equal(fsum(mtcNA, g, wdatNA, na.rm = FALSE), BY(mtcNA, gf, wsum, wdatNA))
+  expect_equal(fsum(mtcNA, g, wdatNA), condan20(BY(mtcNA, gf, wsum, wdatNA, na.rm = TRUE), fill_arg == 2L))
 })
 
 test_that("fsum performs numerically stable", {
